/*

	$Id: namelevel.c,v 1.1 1993/05/07 01:46:19 k-chinen Exp $
*/

#include <stdio.h>

#include <X11/StringDefs.h>
#include <X11/IntrinsicP.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/XawInit.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Scrollbar.h>

#include "namelevelP.h"

#define offset(field) XtOffset(LevelWidget, field)

static XtResource resources[] = {
	/* inherit resource */
	{ XtNheight,XtCWidth,XtRDimension,	sizeof(Dimension),
		offset(core.height),	XtRImmediate, (XtPointer) 200},
	{ XtNwidth,	XtCHeight,XtRDimension,	sizeof(Dimension),
		offset(core.width),	XtRImmediate, (XtPointer) 200},
	{XtNorientation,XtCOrientation,XtROrientation,sizeof(XtOrientation),
		offset(level.orient),	XtRImmediate, (XtPointer) XtEvertical},
	{XtNcallback,XtCCallback,XtRCallback,sizeof(XtCallbackProc),
		offset(level.callback),	XtRImmediate, (XtPointer) NULL},

	/* new resource */
	{ XtNname,	XtCString,	XtRString,		sizeof(String),
		offset(level.name),	XtRImmediate, (XtPointer) "name"},
	{ XtNlevel,	XtCThickness,	XtRInt,		sizeof(int),
		offset(level.level),	XtRImmediate, (XtPointer) 50},
	{ XtNlevelMax,	XtCThickness,	XtRInt,		sizeof(int),
		offset(level.level_max),	XtRImmediate, (XtPointer) 100},
	{ XtNlevelMin,	XtCThickness,	XtRInt,		sizeof(int),
		offset(level.level_min),	XtRImmediate, (XtPointer) 0},
	{ XtNvalueWidth,XtCThickness,	XtRInt,		sizeof(int),
		offset(level.value_width),	XtRImmediate, (XtPointer) 3},
};
#undef offset

static void Initialize();
static void Jump_Proc(Widget w, XtPointer client_data, XtPointer call_data);
static void Scroll_Proc(Widget w, XtPointer client_data, XtPointer call_data);

LevelClassRec levelClassRec = {
	{
		(WidgetClass) & formClassRec,
		"Level",
		sizeof(LevelRec),
		XawInitializeWidgetSet,
		NULL,
		FALSE,
		Initialize,
		NULL,
		XtInheritRealize,
		NULL,
		ZERO,
		resources,
		XtNumber(resources),
		NULLQUARK,
		TRUE,
		TRUE,
		TRUE,
		FALSE,
		NULL,
		XtInheritResize,
		XtInheritExpose,
		NULL,
		NULL,
		XtInheritSetValuesAlmost,
		NULL,
		NULL,
		XtVersion,
		NULL,
		NULL,
		XtInheritQueryGeometry,
		XtInheritDisplayAccelerator,
		NULL
	},
	{
		XtInheritGeometryManager,
		XtInheritChangeManaged,
		XtInheritInsertChild,
		XtInheritDeleteChild,
		NULL
	},
	{
		NULL,
		0,
		sizeof(LevelConstraintsRec),
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		XtInheritLayout
	},
	{
		0,
	},
};

WidgetClass levelWidgetClass = (WidgetClass) &levelClassRec;

static void Initialize(LevelWidget request, LevelWidget new)
{
	char tmp[256];

	Widget  name, value, bar;
	Dimension WW,HH,ww,hh;
	float percent;

	sprintf(tmp,"%*d",new->level.value_width,new->level.level);

	name = XtVaCreateManagedWidget( "levelvalue",
				labelWidgetClass,	(Widget)new,
				XtNlabel,			new->level.name,
				XtNborderWidth,		0,
				NULL);

	value = XtVaCreateManagedWidget( "levelvalue",
				labelWidgetClass,	(Widget)new,
				XtNlabel,			tmp,
				XtNborderWidth,		0,
				NULL);

	bar = XtVaCreateManagedWidget( "bar",
				scrollbarWidgetClass,	(Widget)new,
				XtNborderWidth,		1,
				XtNlength,			30,
				/* XtNminimumThumb,	30, */
				NULL);

	if((char*)new->level.orient==XtEhorizontal) {
		XtVaGetValues(name, XtNheight, &HH, NULL);
		XtVaGetValues(name, XtNwidth, &WW, NULL);
		XtVaSetValues( value,
			XtNfromHoriz, 	name,
			NULL);
		XtVaGetValues(value, XtNheight, &hh, NULL);
		XtVaGetValues(value, XtNwidth, &ww, NULL);
		XtVaSetValues( bar,
			XtNfromHoriz, 	value,
			XtNorientation, XtorientHorizontal,
			XtNheight,		HH>hh?HH:hh,
			XtNwidth,		new->core.width-ww-WW,
			NULL);
	}
	else {
		XtVaGetValues(name, XtNheight, &HH, NULL);
		XtVaGetValues(name, XtNwidth, &WW, NULL);
		XtVaSetValues( value,
			XtNfromVert, 	name,
			NULL);
		XtVaGetValues(value, XtNheight, &hh, NULL);
		XtVaGetValues(value, XtNwidth, &ww, NULL);
		XtVaSetValues( bar,
			XtNfromVert, 	value,
			XtNorientation, XtorientVertical,
			XtNheight,		new->core.height-hh-HH,
			XtNwidth,		WW>ww?WW:ww,
			NULL);
	}

    XtAddCallback(bar,	XtNjumpProc, Jump_Proc,	NULL);
    XtAddCallback(bar,	XtNscrollProc, Scroll_Proc,	NULL);
	percent = ((float)new->level.level)/
			(new->level.level_max - new->level.level_min);
	/*
	printf("level=%d,percent=%f\n", new->level.level, percent);
	*/

	XawScrollbarSetThumb(bar, percent-0.02>0?percent-0.02:0.0, 0.04);
}



static void Jump_Proc(Widget w, XtPointer client_data, XtPointer call_data)
{
	LevelWidget level;
	LabelWidget value;
    /* char tmp[128]="X"; */
    char tmp[128];
    float percent = *(float*) call_data;

	level = (LevelWidget)(w->core.parent);
	value = (LabelWidget)level->composite.children[1];

	level->level.level =
		(int)(percent*(level->level.level_max-level->level.level_min));

	/*
	printf("level=%d,percent=%f\n", level->level.level, percent);
	*/
    sprintf(tmp,"%*d",level->level.value_width,level->level.level);

	XtVaSetValues((Widget)value, XtNlabel, tmp, NULL);
	XawScrollbarSetThumb(w, percent-0.02>0?percent-0.02:0.0, 0.04);

	XtCallCallbacks((Widget)level, XtNcallback,
		(XtPointer) (level->level.level));
}

static void Scroll_Proc(Widget w, XtPointer client_data, XtPointer call_data)
{
	LevelWidget level;
	LabelWidget value;
    /* char tmp[128]="X"; */
    char tmp[128];
	int	position = (int)call_data;
	int wide;
    float percent;

	level = (LevelWidget)(w->core.parent);
	value = (LabelWidget)level->composite.children[1];

	wide=level->level.level_max - level->level.level_min;

	if(position<=0) level->level.level += (int)(0.1*wide);
	else 			level->level.level -= (int)(0.1*wide);

	if(level->level.level<=level->level.level_min) {
		level->level.level=level->level.level_min;
	}
	if(level->level.level>level->level.level_max) {
		level->level.level=level->level.level_max;
	}
	percent = ((float)level->level.level)/wide;
	/*
	printf("level=%d,percent=%f\n", level->level.level, percent);
	*/
	XawScrollbarSetThumb(w, percent-0.02>0?percent-0.02:0.0, 0.04);

    sprintf(tmp,"%*d",level->level.value_width,level->level.level);

	XtVaSetValues((Widget)value, XtNlabel, tmp, NULL);

	XtCallCallbacks((Widget)level, XtNcallback,
		(XtPointer) (level->level.level));
}
