/*

    SP: Paint progam
        ----- Sample for Xt -----

        by  k-chinen@is.aist-nara.ac.jp , 1994

    $Id$

	Reference:	mit,		clients/xwd/dsimple.c
				contrib,	clients/xsnap/xsnap.c
*/

#include "sp.h"

int pick_window(Window *w, int *x, int *y)
{
	XEvent event;
	Cursor cursor;
	Status status;
	Window target = None;
	int buttons = 0;

	cursor = XCreateFontCursor(dpy, XC_crosshair);

	status = XGrabPointer(dpy, root, False,
			ButtonPressMask, GrabModeSync,
			GrabModeAsync, root, cursor, CurrentTime);

	if (status != GrabSuccess) {
		printf("select_window: cannot grab pointer\n");
		return 1; 
	}

	while (buttons == 0) {
		XAllowEvents(dpy, SyncPointer, CurrentTime);
		XWindowEvent(dpy, root, ButtonPressMask, &event);
		switch (event.type) {
		case ButtonPress:
			if(event.xbutton.button == 1) {
				target = event.xbutton.subwindow; 
				if (target == None)
					target = root;
				buttons++;
			}
			else {
				buttons = -1;
			}
      		break;
    	}
  	}

  	XUngrabPointer(dpy, CurrentTime);
	if(buttons<0) {
		return 1;
	}

/*
	printf("xcut: target = %d\n",target);
	printf("xcut: pos = (%d,%d)\n",
		event.xbutton.x,event.xbutton.y);
	printf("xcut: root pos = (%d,%d)\n",
		event.xbutton.x_root,event.xbutton.y_root);
*/

	*w = target;
	*x = event.xbutton.x;
	*y = event.xbutton.y;

	return 0;
}

int pick_color()
{
	long	val;
	XImage *image;
	Status status;
	Window target;
	int		x,y;
	XWindowAttributes xwa;
	
	if(pick_window(&target,&x,&y)) {
		/*
		printf("pick_color: cannot grab pointer\n");
		*/
	}
	else {
		/*
		printf("pick_color: target = %d\n",target);
		*/
	}

	status = XGetWindowAttributes(dpy, target, &xwa);

/*
	printf("child window pos = (%d,%d)\n",xwa.x,xwa.y);
	printf("child window size = (%d,%d)\n",xwa.width,xwa.height);
	printf("pointer pos in child window= (%d,%d)\n",x-xwa.x,y-xwa.y);
*/

	image = XGetImage(dpy, target,
		x-xwa.x, y-xwa.y, 1, 1, AllPlanes, ZPixmap);
	val = XGetPixel(image, 0,0);
	XDestroyImage(image);

/*
	printf("pixel of such pos. %d .\n",val);
*/

	return val;
}


void draw_box(a_display, a_window, a_gc, x1, y1, x2, y2)
GC a_gc;
Window a_window;
Display *a_display;
int x1, y1, x2, y2;
{
    XSegment segments[4];
    segments[0].x1 = (short)x1;
    segments[0].y1 = (short)y1;
    segments[0].x2 = (short)x1;
    segments[0].y2 = (short)y2;

    segments[1].x1 = (short)x1;
    segments[1].y1 = (short)y1;
    segments[1].x2 = (short)x2;
    segments[1].y2 = (short)y1;

    segments[2].x1 = (short)x2;
    segments[2].y1 = (short)y2;
    segments[2].x2 = (short)x1;
    segments[2].y2 = (short)y2;

    segments[3].x1 = (short)x2;
    segments[3].y1 = (short)y2;
    segments[3].x2 = (short)x2;
    segments[3].y2 = (short)y1;

    XDrawSegments(a_display, a_window, a_gc, segments, 4);
}

Window create_event_window(a_display,a_window, init_cursor)
Display *a_display;
Window a_window;
{
    XSetWindowAttributes xswa;
    unsigned long xswa_valuemask;
    Window root_win;
    Window event_window;
    unsigned int win_width, win_height;
    unsigned int  win_border_width, win_depth;
    int win_x, win_y;

    /* get the geometry of the window  */

    XGetGeometry(a_display, a_window, &root_win, &win_x, &win_y,
        &win_width, &win_height, &win_border_width, &win_depth);

    /* make an input only window to get events from  */

    xswa.cursor = init_cursor;
    xswa.override_redirect = True;
    xswa.event_mask = ButtonPressMask | ButtonReleaseMask | Button1MotionMask;
    xswa_valuemask = CWCursor | CWOverrideRedirect | CWEventMask;
    event_window = XCreateWindow(a_display, a_window, win_x, win_y, win_width,
        win_height, 0, 0, InputOnly, CopyFromParent,
        xswa_valuemask, &xswa);
    return(event_window);
}


int pick_area(Window *w, int *x, int *y, int *wid, int *ht)
{
	XEvent event;
	Cursor cursor;
	Status status;
	Window target = None;
	int buttons = 0, drag = 0;
	int	ax,ay,bx,by;
	Window ewin;

	cursor = XCreateFontCursor(dpy, XC_crosshair);

	ewin = create_event_window(dpy, root, cursor);
	XMapRaised(dpy, ewin);

	status = XGrabPointer(dpy, ewin, False,
			ButtonPressMask|ButtonReleaseMask|Button1MotionMask, GrabModeSync,
			GrabModeAsync, ewin, cursor, CurrentTime);

	if (status != GrabSuccess) {
		printf("select_window: cannot grab pointer\n");
		return 1; 
	}

	/*
	XSetForeground(dpy,xorGC,white);
	*/
	XSetForeground(dpy,xorGC,black);

	while (buttons == 0) {
		XAllowEvents(dpy, SyncPointer, CurrentTime);
		XWindowEvent(dpy, ewin,
			ButtonPressMask|ButtonReleaseMask|Button1MotionMask, &event);
		switch (event.type) {
		case ButtonPress:
			if(event.xbutton.button == 1) {
				/*
				target = event.xbutton.subwindow; 
				if (target == None)
					target = root;
				*/
				ax = event.xbutton.x;
				ay = event.xbutton.y;
				bx = ax;
				by = ay;

				drag = 1;
				draw_box(dpy,root,xorGC,ax,ay,bx,by);

				XFlush(dpy);
			}
			else {
				buttons = -1;
			}
      		break;
		case ButtonRelease:
			if(event.xbutton.button == 1) {
				draw_box(dpy,root,xorGC,ax,ay,bx,by);
				bx = event.xbutton.x;
				by = event.xbutton.y;
				buttons = 1;

				XFlush(dpy);
			}
			break;
		case MotionNotify:
			if(drag) {
				/*
				putchar('*');
				*/
				fflush(stdout);

				draw_box(dpy,root,xorGC,ax,ay,bx,by);
				bx = event.xmotion.x;
				by = event.xmotion.y;
				draw_box(dpy,root,xorGC,ax,ay,bx,by);

				XFlush(dpy);
			}
			break;
    	}
  	}

  	XUngrabPointer(dpy, CurrentTime);
    XDestroyWindow(dpy, ewin);

	if(buttons<0)
		return 1;
		
	if(ax<bx) {
		*x = ax;
		*wid = bx-ax;
	}
	else {
		*x = bx;
		*wid = ax-bx;
	}

	if(ay<by) {
		*y = ay;
		*ht = by-ay;
	}
	else {
		*y = by;
		*ht = ay-by;
	}

/*
	printf("xcut: area = (%d,%d) - (%d,%d)\n",*x,*y,*wid,*ht);
*/

	return 0;
}


int get_pos(Window w, int *x, int *y)
{
	Status status;
	int		r=0,buttons=0;
	XEvent	event;
	XWindowAttributes xwa;
	XSetWindowAttributes xswa;

	status = XGetWindowAttributes(dpy, w, &xwa);
		
	/*
	if (!status) {
		printf("NO  !!!\n");
		return 1; 
	}
	printf("FINE!!!\n");
	*/

	XSelectInput(dpy, w, ButtonPressMask| ButtonReleaseMask);

	do {
		XNextEvent(dpy,&event);
		switch (event.type) {
		case ButtonPress:
			if(event.xbutton.button == 1) {
				*x = event.xbutton.x;
				*y = event.xbutton.y;
				r = 1;
			}
			else {
				buttons = -1;
			}
      		break;
		case ButtonRelease:
			if(r && event.xbutton.button == 1) 
				buttons = 1;
			break;
		default:
			continue;
		}
	} while(buttons==0);

	xswa.event_mask = xwa.your_event_mask;
	XChangeWindowAttributes(dpy,w,CWEventMask,&xswa);

	if(buttons>0)
		return 0;
	else
		return 1;
}
