/*

    SP: Paint progam
        ----- Sample for Xt -----

        by  k-chinen@is.aist-nara.ac.jp , 1994

	$Id$
*/

#include "sp.h"


#define _DEBUG_EPS



static eps_comment(FILE *fp, char *name, int w, int h)
{
	fprintf(fp,
"%%!PS-Adobe-2.0 EPSF-2.0\n\
%%%%BoundingBox: 0 0 %d %d\n\
%%%%Creator: SP (Paint program by k-chinen@is.aist-nara.ac.jp)\n\
%%%%Title: %s\n\
%%%%EndComments\n\
\n\
%%%%Page: 1 1\n\
\n\
10 10 translate\n\
1 1 scale\n\n"
	,w+20 , h+20, name
);
}

static eps_trailer(FILE *fp)
{
	fprintf(fp,"showpage\n");
	/* */
	fprintf(fp,"%%%%PageTrailer\n");
	fprintf(fp,"%%%%Trailer\n");
	fprintf(fp,"%% END of SP-output\n");
	/* */
}


void write_colorepsfile(char *name, Drawable dp, unsigned int w, unsigned int h)
{
	FILE *fp;
	int z,x,y;
	XImage *image;
	XColor c,tbl[256];
	unsigned long p;


#ifdef DEBUG_EPS
	fprintf(stderr,"COLOR EPS\n");
#endif

	if((fp=fopen(name,"w"))==NULL) {
		fprintf(stderr,"Cannot open %s\n",name);
		return;
	}

#ifdef DEBUG_EPS
	fprintf(stderr,"Write %s\n",name);
#endif

	eps_comment(fp,name,w,h);

	fprintf(fp,"/pics %d string def\n",6*w);

    image = XGetImage(dpy, dp,
			0, 0, w, h, AllPlanes, ZPixmap);
	fprintf(fp,"%d %d 8 matrix {currentfile pics readhexstring pop}\n",
		w,h);
	fprintf(fp,"false 3 colorimage\n");

	for(z=0;z<256;z++) {
		tbl[z].pixel = z;
		XQueryColor(dpy, cmap, &tbl[z]);
	}

	for(y=h-1;y>=0;y--) {
		for(x=0;x<w;x++) {
			p = XGetPixel(image, x,y);
			fprintf(fp,"%02x%02x%02x",
				tbl[p].red/256,tbl[p].green/256,tbl[p].blue/256);
		}
		fprintf(fp,"\n");
	}

	eps_trailer(fp);

	XDestroyImage(image);

	fclose(fp);
}

void write_monoepsfile(char *name, Drawable dp, unsigned int w, unsigned int h)
{
	FILE *fp;
	int z,x,y;
	XImage *image;
	XColor c,tbl[256];
	unsigned long p;
	int max_color;

#ifdef DEBUG_EPS
	fprintf(stderr,"MONO EPS\n");
#endif

	if((fp=fopen(name,"w"))==NULL) {
		fprintf(stderr,"Cannot open %s\n",name);
		return;
	}

#ifdef DEBUG_EPS
	fprintf(stderr,"Write %s\n",name);
#endif

	eps_comment(fp,name,w,h);

	fprintf(fp,"/pics %d string def\n",2*w);

    image = XGetImage(dpy, dp,
			0, 0, w, h, AllPlanes, ZPixmap);
	fprintf(fp,"%d %d 8 matrix {currentfile pics readhexstring pop}\n",
		w,h);
	fprintf(fp,"image\n");

	switch(vis->class) {
	case StaticGray:
	case GrayScale:
		max_color = 2;
		break;
	default:
		max_color = 256;
		break;
	}
	for(z=0;z<max_color;z++) {
		tbl[z].pixel = z;
		XQueryColor(dpy, cmap, &tbl[z]);
	}

	for(y=h-1;y>=0;y--) {
		for(x=0;x<w;x++) {
			p = XGetPixel(image, x,y);
			fprintf(fp,"%02x",(tbl[p].red+tbl[p].green+tbl[p].blue)/(256*3));
		}
		fprintf(fp,"\n");
	}

	eps_trailer(fp);

	XDestroyImage(image);

	fclose(fp);
}
